@@ -5,7 +5,7 @@
 # This file is part of the `plugy` python module
 #
 # Copyright
-# 2018-2021
+# 2018-2022
 # EMBL & Heidelberg University
 #
 # Author(s): Dénes Türei (turei.denes@gmail.com)
@@ -19,8 +19,11 @@
 # Webpage: https://github.com/saezlab/plugy
 #
 
+from typing import Iterable, Optional
+from numbers import Number
 
 import sys
+import os
 import logging
 import collections
 import importlib as imp
@@ -30,13 +33,14 @@ from dataclasses import dataclass, field
 
 import numpy as np
 import pandas as pd
-import scipy.stats as stats
-import statsmodels.stats.multitest as statsmod
 
 import matplotlib as mpl
 import matplotlib.pyplot as plt
 import seaborn as sns
 
+# to output text instead of paths
+plt.rcParams['svg.fonttype'] = 'none'
+
 from .data.config import PlugyConfig
 from .data.bd import ChannelMap, PlugSequence
 from .data.pmt import PmtData
@@ -199,6 +203,7 @@ class PlugExperiment(object):
 
         self.drug_combination_analysis()
         self.close_figures()
+        self.stats_table_tsv()
 
 
     def _set_workflow_param(self):
@@ -353,7 +358,7 @@ class PlugExperiment(object):
 
         self.sample_data = self.get_sample_data()
         self.append_experiments()
-        self.sample_statistics = self.calculate_statistics()
+        self.sample_statistics = self.stats()
 
 
     def append_experiments(self, experiments: list = None):
@@ -382,7 +387,15 @@ class PlugExperiment(object):
             self.plug_data.append(*exp)
 
 
-    def plot_pmt_data(self, width = 300, **kwargs):
+    def plot_pmt_data(
+            self,
+            width: Number = 300,
+            xmin: Optional[Number] = None,
+            xmax: Optional[Number] = None,
+            ymax: Optional[Number] = None,
+            label: Optional[str] = None,
+            **kwargs
+        ):
 
         qc_dir = self.ensure_qc_dir()
 
@@ -395,6 +408,10 @@ class PlugExperiment(object):
             **kwargs,
         )
 
+
+        pmt_overview_ax.set_xlim(xmin, xmax)
+        pmt_overview_ax.set_ylim(0.0, ymax)
+
         self.plug_data.highlight_plugs(
             axes = pmt_overview_ax,
             below_peak = False,
@@ -407,11 +424,17 @@ class PlugExperiment(object):
             misc.add_git_hash_caption(pmt_overview_fig)
 
         pmt_overview_fig.tight_layout()
-        png_path = qc_dir.joinpath(f"pmt_overview.png")
-        pmt_overview_fig.savefig(png_path)
+        fmt = (
+            'png'
+                if self.config.pmt_overview_force_png else
+            self.config.figure_export_file_type
+        )
+        label = f'_{label}' if label else ''
+        path = qc_dir.joinpath(f"pmt_overview{label}.{fmt}")
+        pmt_overview_fig.savefig(path)
         plt.clf()
 
-        module_logger.info(f"Plotted PMT data to {png_path}")
+        module_logger.info(f"Plotted PMT data to `{path}`.")
 
 
     def plot_plug_sequence(self):
@@ -520,11 +543,13 @@ class PlugExperiment(object):
         qc_issues = []
         qc_dir = self.ensure_qc_dir()
 
+        # 1: drift of the medium only control
         self.plot_medium_control_trends()
 
+        # 2: raw PMT data
         self.plot_pmt_data()
 
-        # Plotting plug numbers
+        # 3: per sample plug counts
         plug_count_hist_fig, plug_count_hist_ax = plt.subplots()
         plug_count_hist_ax = self.plot_plug_count_hist(
             axes = plug_count_hist_ax,
@@ -540,9 +565,10 @@ class PlugExperiment(object):
         )
         plt.clf()
 
+        # 4: plug lengths (sizes)
         self.plot_length_bias()
 
-        # Plotting contamination
+        # 5: barcode dye contamination of sample plugs
         contamination = self.get_contamination()
         if contamination.mean() > self.config.contamination_threshold:
             msg = (
@@ -572,21 +598,34 @@ class PlugExperiment(object):
 
             contamination_hist_ax.set_title("Relative barcode contamination")
 
-        contamination_hist_ax.set_xlabel(
-            "Barcode control ratio\n"
-            r"$\left[\frac{\overline{barcode_{median}}}"
-            r"{\overline{control_{median}}}\right]$"
-        )
-        contamination_hist_ax.set_ylabel("Number of sample plugs")
-        misc.add_git_hash_caption(contamination_hist_fig)
-        contamination_hist_fig.tight_layout()
-        contamination_hist_fig.savefig(
-            qc_dir.joinpath(
-                f"contamination_hist.{self.config.figure_export_file_type}"
+        xlab = "Barcode to control ratio"
+
+        if self.config.figure_export_file_type == 'png':
+
+            xlab += (
+                r"\n$\left[\frac{\overline{barcode_{median}}}"
+                r"{\overline{control_{median}}}\right]$"
             )
+
+        contamination_hist_ax.set_xlabel(xlab)
+
+        contamination_hist_ax.set_ylabel("Number of sample plugs")
+
+        if self.config.plot_git_caption:
+
+            misc.add_git_hash_caption(contamination_hist_fig)
+
+        contamination_hist_fig.tight_layout()
+
+        path = qc_dir.joinpath(
+            f"contamination_hist.{self.config.figure_export_file_type}"
         )
+        contamination_hist_fig.savefig(path)
         plt.clf()
 
+        module_logger.info(f"Saving contamination histogram to `{path}`.")
+
+        # 6: channel vs. channel scatterplot matrix
         contamination_fig, contamination_ax = plt.subplots(
             2, 3,
             sharex = "all",
@@ -634,22 +673,42 @@ class PlugExperiment(object):
             misc.add_git_hash_caption(contamination_fig)
 
         contamination_fig.tight_layout()
-        contamination_fig.savefig(
-            qc_dir.joinpath(
-                f"contamination.{self.config.figure_export_file_type}"
-            )
+
+        path = qc_dir.joinpath(
+            f"contamination.{self.config.figure_export_file_type}"
         )
+        contamination_fig.savefig(path)
         plt.clf()
 
-        self.plot_control()
-        self.plot_sample_cycles()
+        module_logger.info(f"Saving contamination scatter plots to `{path}`.")
 
+        # 7: composite single channel insights figures
+        self.univar_overviews()
+
+        # 8: raw data in sample vs. cycle grid
+        self.plot_samples_cycles()
+
+        # 9: grid of plug length (size) vs. channels
         self.length_grid()
+
+        # 10: density plots of plug lengths (sizes)
         self.length_density()
+        self.length_boxplot()
+
+        # 11: density plots of plug volumes
         self.volume_density()
+        self.volume_boxplot()
+
+        # 12: violin plots of standard deviations of samples:
+        # readout, readout:control and medium corrected readout:control
         self.sample_sd_violin()
+
+        # 13: printing coefficients of variance into the log
         self.report_cv()
 
+        # 14: heatmap matrix
+        self.heatmap_matrix()
+
         plt.close('all')
 
         qc_successful = not qc_issues
@@ -703,12 +762,12 @@ class PlugExperiment(object):
         )
         media_control_ax[0] = (
             self.plug_data.plot_medium_control_trends(
-                axes = media_control_ax[0],
+                ax = media_control_ax[0],
             )
         )
         media_control_ax[1] = (
             self.plug_data.plot_medium_control_trends(
-                axes = media_control_ax[1],
+                ax = media_control_ax[1],
                 by_sample = True,
             )
         )
@@ -750,145 +809,301 @@ class PlugExperiment(object):
             module_logger.error('Failed to plot length bias')
 
 
-    def plot_control(self):
+    def heatmap_matrix(self, **kwargs):
         """
-        Creates a composite figure to investigate if the control channel
-        shows a bias with other variables or correlation with the readout
-        channel.
+        Creates a composite figure to overview the medians of conditions
+        by cycle vs. channel or derived variable.
+        """
+
+        grid = self.plug_data.heatmap_matrix(**kwargs)
+
+        self._plot_base(grid, 'heatmap_matrix', 'heatmap matrix')
+
+
+    def univar_overview(
+            self,
+            var: str = None,
+            var2: str = 'readout_peak_median',
+            var3: str = 'control_peak_median',
+        ):
+        """
+        Creates a composite figure to investigate one variable and discover
+        potential bias or drift across time, cycles, samples, or in relation
+        to another variable. It is especially suitable to see if the control
+        channel shows a bias with other variables or correlation with the
+        readout channel.
+
+        Args:
+            var:
+                A variable name in the plug or sample data frame.
+            var2:
+                Another variable: it will be used only for one panel,
+                a scatter plot ``var`` vs. ``var2``.
         """
 
         self.seaborn_setup(font_scale = self.config.font_scale * .7)
 
         qc_dir = self.ensure_qc_dir()
 
-        # Plotting control
-        control_fig = plt.figure(
-            figsize = (20, 10),
-            constrained_layout = False,
-        )
-        control_fig_gs = control_fig.add_gridspec(nrows = 2, ncols = 4)
-        control_ax_sample_dist = control_fig.add_subplot(control_fig_gs[0, :])
-        control_ax_control_regression = (
-            control_fig.add_subplot(control_fig_gs[1, 0])
-        )
-        control_ax_cycle_dist = control_fig.add_subplot(control_fig_gs[1, 1])
-        control_ax_readout_correlation = (
-            control_fig.add_subplot(control_fig_gs[1, 2])
+        fig = self.plug_data.univar_overview(
+            var = var,
+            var2 = var2,
+            var3 = var3,
         )
+        fig.tight_layout()
 
-        control_ax_control_regression = (
-            self.plug_data.plot_control_regression(
-                control_ax_control_regression
-            )
-        )
-        control_ax_cycle_dist = self.plug_data.plot_control_cycle_dist(
-            control_ax_cycle_dist
-        )
-        control_ax_sample_dist = self.plug_data.plot_control_sample_dist(
-            control_ax_sample_dist
-        )
-        control_ax_readout_correlation = (
-            self.plug_data.plot_control_readout_correlation(
-                control_ax_readout_correlation
-            )
-        )
-
-        grid = self.plug_data.plot_compound_heatmap(
-            column_to_plot = 'control_peak_median',
-        )
-
-        control_fig.axes[-1] = grid.axes.flat[0]
-
-        control_fig.tight_layout()
         if self.config.plot_git_caption:
-            misc.add_git_hash_caption(control_fig)
-        control_fig.savefig(
-            qc_dir.joinpath(
-                f"control_fluorescence.{self.config.figure_export_file_type}"
-            )
+
+            misc.add_git_hash_caption(fig)
+
+        path = qc_dir.joinpath(
+            f'{var}_overview.{self.config.figure_export_file_type}'
         )
+        fig.savefig(path)
         plt.clf()
 
+        module_logger.info(f"Saving overview plot of `{var}` to `{path}`.")
+
         self.seaborn_setup()
 
 
-    def plot_sample_cycles(self, **kwargs):
+    def univar_overviews(
+            self,
+            variables: Iterable[str] | None = None,
+            var2: str | None = None,
+        ):
+        """
+        Calls ``univar_overview`` for each channel and the derived variables.
+        It means creating a composite figure for each of these variables.
+        """
+
+        default_vars = misc.CHANNEL_VARS
+        variables = misc.to_tuple(variables) or default_vars
+
+        for var in variables:
+
+            self.univar_overview(var = var)
+
+
+    def plot_samples_cycles(
+            self,
+            label: Optional[str] = None,
+            ymax: Optional[Number] = None,
+            **kwargs
+        ):
+        """
+        Creates a plot with raw data for the individual samples and cycles.
+
+        Args:
+            label:
+                A custom label to be included in the file name. Useful if
+                plotting a subset of samples.
+        """
 
         self.seaborn_setup(font_scale = self.font_scale * .7)
 
         # Plotting PMT overview
         try:
+
             qc_dir = self.ensure_qc_dir()
+
             sample_cycle_fig, sample_cycle_ax = (
-                self.plug_data.plot_sample_cycles(**kwargs)
+                self.plug_data.plot_samples_cycles(
+                    ylim = (None, ymax),
+                    **kwargs
+                )
             )
 
             if self.config.plot_git_caption:
 
                 misc.add_git_hash_caption(sample_cycle_fig)
 
-            sample_cycle_fig.savefig(
-                qc_dir.joinpath('sample_cycle_overview.png')
+            ftype = (
+                self.config.figure_export_file_type
+                    if 0 < len(kwargs.get('samples', ())) < 9 else
+                'png'
             )
+
+            label = f'_{label}' if label else ''
+            fname = f'sample_cycle_overview{label}.{ftype}'
+            path = qc_dir.joinpath(fname)
+            sample_cycle_fig.savefig(path)
+            module_logger.info(f'Saved figure to `{path}`')
+
             plt.clf()
+
         except:
+
             traceback.print_exc(file = sys.stdout)
-            module_logger.error('Failed to plot sample cycles')
+            module_logger.error('Failed to plot samples vs. cycles.')
 
         self.seaborn_setup()
 
 
-    def calculate_statistics(self) -> pd.DataFrame:
+    def stats(
+            self,
+            cycle: int | None = None,
+            extra_cols: list[str] = None,
+        ) -> pd.DataFrame:
         """
-        Calculates statistics
+        Calculates statistics for each sample: means, standard deviations,
+        Wilcoxon tests, their adjusted p-values and significances.
+
+        Args:
+            cycle:
+                Use only this experiment cycle. If None, all cycles will be
+                used.
+            extra_cols:
+                Keep also these columns, calculate their means and standard
+                deviations.
+
+        Returns:
+            A data frame with the sample level statistics, with compounds
+            in row multi index.
         """
-        module_logger.info("Calculating statistics")
-        media_data = self.plug_data.medium_only()
-        compound_data = self.plug_data.sample_df[
-            ~self.plug_data.sample_df.isin(media_data)
-        ].dropna()
 
-        group_columns = ["compound_a", "compound_b", "name"]
-        sample_stats = compound_data.groupby(by = group_columns)[
-            self.config.readout_analysis_column
-        ].agg([np.mean, np.std])
+        return self.plug_data.stats(cycle = cycle, extra_cols = extra_cols)
 
-        p_values = list()
 
-        for combination, values in compound_data.groupby(by = group_columns):
+    def stats_table(
+            self,
+            by_cycle: bool = True,
+            exp_summary: bool = True,
+            extra_cols: bool | list[str] = True,
+            sort_by: list[str] | None = None,
+            multi_index: bool = True,
+        ) -> pd.DataFrame:
+        """
+        A large table of statistics.
 
-            p_values.append(
-                stats.ranksums(
-                    x = values[self.config.readout_analysis_column],
-                    y = media_data[self.config.readout_analysis_column]
-                )[1]
-            )
+        Args:
+            by_cycle:
+                Include statistics for each cycle.
+            exp_summary:
+                Include statistics for the entire experiment.
+            extra_cols:
+                Include extra columns, besides the defaults. Alternatively,
+                a list of column names that overrides the extra columns.
+            sort_by:
+                Sort the conditions by these column names. For descending
+                order prefix the column name with a hat (^). Cycle will
+                always be the first column to sort by.
+            multi_index:
+                Keep conditions in multi index. If `False`, the multi index
+                will be turned into regular columns.
 
-        sample_stats = sample_stats.assign(pval = p_values)
-        significance, p_adjusted, _, alpha_corr_bonferroni = (
-            statsmod.multipletests(
-                pvals = sample_stats.reset_index().pval,
-                alpha = self.config.alpha,
-                method = "bonferroni",
-            )
-        )
-        sample_stats = sample_stats.assign(
-            p_adjusted = p_adjusted,
-            significant = significance,
+        Return:
+            A data frame of the statistics with conditions in row multi index.
+        """
+
+        args = locals()
+        args.pop('self')
+
+        return self.plug_data.stats_table(**args)
+
+
+    def _stats_table_export(
+            self,
+            path: str | None = None,
+            ftype: str = 'tsv',
+            by_cycle: bool = True,
+            exp_summary: bool = True,
+            extra_cols: bool | list[str] = True,
+            sort_by: list[str] | None = None,
+            multi_index: bool = True,
+            **kwargs
+        ):
+        """
+        Export a table of statistics.
+
+        Args:
+            path:
+                Path or file name. Default directory is *results*, default
+                file name is *results/<input-name>_stats-table.<ftype>*.
+            ftype:
+                File type, such as "csv", "tsv", "xlsx", or any other type
+                supported by ``pandas``.
+            kwargs:
+                Passed to pandas export (``to_...``) functions.
+        """
+
+        table = self.stats_table(
+            by_cycle = by_cycle,
+            exp_summary = exp_summary,
+            extra_cols = extra_cols,
+            sort_by = sort_by,
+            multi_index = multi_index,
         )
 
-        # Renaming columns to avoid shadowing mean function
-        sample_stats.columns = [
-            "mean_z_score",
-            "std_z_score",
-            "pval",
-            "p_adjusted",
-            "significant",
-        ]
+        defaults = dict(
+            fname_suffix = 'stats-table',
+            sheet_name = f'stats__{self.pmt_data.input_file.stem}',
+            label = 'large stats table',
+        )
+        defaults.update(kwargs)
 
-        # # Reindex based on sample_df from plug.PlugData object
-        # sample_stats = sample_stats.reindex(self.plug_data.sample_df.name.unique())
-        return sample_stats
+        self._export(df = table, path = path, ftype = ftype, **defaults)
+
+
+    def stats_table_tsv(
+            self,
+            path: str | None = None,
+            by_cycle: bool = True,
+            exp_summary: bool = True,
+            extra_cols: bool | list[str] = True,
+            sort_by: list[str] | None = None,
+            **kwargs
+        ):
+        """
+        Export a table of statistics to TSV.
+
+        Args:
+            path:
+                Path or file name. Default directory is *results*, default
+                file name is *results/<input-name>_stats-table.tsv*.
+            kwargs:
+                Passed to ``pandas.DataFrame.to_csv``.
+
+        The rest of the arguments are described at ``stats_table``.
+        """
+
+        args = locals()
+        args.pop('self')
+        args.pop('kwargs')
+        args['ftype'] = 'tsv'
+
+        self._stats_table_export(**args, **kwargs)
+
+
+    def stats_table_excel(
+            self,
+            path: str | None = None,
+            by_cycle: bool = True,
+            exp_summary: bool = True,
+            extra_cols: bool | list[str] = True,
+            sort_by: list[str] | None = None,
+            multi_index: bool = True,
+            **kwargs
+        ):
+        """
+        Export a table of statistics to XLSX.
+
+        Args:
+            path:
+                Path or file name. Default directory is *results*, default
+                file name is *results/<input-name>_stats-table.xlsx*.
+            kwargs:
+                Passed to ``pandas.DataFrame.to_excel``.
+
+        The rest of the arguments are described at ``stats_table``.
+        """
+
+        args = locals()
+        args.pop('self')
+        args.pop('kwargs')
+        args['ftype'] = 'xlsx'
+
+        self._stats_table_export(**args, **kwargs)
 
 
     def drug_combination_analysis(self):
@@ -957,15 +1172,13 @@ class PlugExperiment(object):
 
             if sample != "Cell Control":
 
-                if statistics.significant[idx]:
-
-                    this_ax.annotate(
-                        "*",
-                        xy = (idx, y_max),
-                        xycoords = "data",
-                        textcoords = "data",
-                        ha = "center",
-                    )
+                this_ax.annotate(
+                    statistics.stars[idx],
+                    xy = (idx, y_max),
+                    xycoords = "data",
+                    textcoords = "data",
+                    ha = "center",
+                )
 
         if not by_cycle:
 
@@ -999,10 +1212,10 @@ class PlugExperiment(object):
         statistics = self.sample_statistics.reset_index()
         statistics = statistics.drop("name", axis = 1)
         statistics = statistics.set_index(["compound_a", "compound_b"])
-        statistics = statistics.significant
+        statistics = statistics.stars
 
-        grid = self.plug_data.plot_compound_heatmap(
-            column_to_plot = self.config.readout_analysis_column,
+        grid = self.plug_data.samples_heatmap(
+            var = self.config.readout_analysis_column,
             annotation_df = statistics,
             by_cycle = by_cycle,
         )
@@ -1037,7 +1250,18 @@ class PlugExperiment(object):
         self._plot_base(grid, 'lengths_density', 'plug length density plots')
 
 
-    def volume_density(self, flow_rate: float = 800.):
+    def length_boxplot(self):
+        """
+        Boxplots about plug lengths. Three panels are created, with barcode
+        plugs, sample plugs and both togther.
+        """
+
+        grid = self.plug_data.size_density(boxplot = True)
+
+        self._plot_base(grid, 'lengths_boxplot', 'length boxplots')
+
+
+    def volume_density(self, flow_rate: float | None = None):
         """
         Density plots and histograms about plug volumes. Three panels are
         created, with barcode plugs, sample plugs and both togther.
@@ -1055,6 +1279,25 @@ class PlugExperiment(object):
         self._plot_base(grid, 'volumes_density', 'plug volume density plots')
 
 
+    def volume_boxplot(self, flow_rate: float | None = None):
+        """
+        Boxplots about plug volumes. Three panels are created, with barcode
+        plugs, sample plugs and both togther.
+
+        Args:
+            flow_rate (float): The flow rate used at the data acquisition
+                in microlitres per hour.
+        """
+
+        grid = self.plug_data.size_density(
+            boxplot = True,
+            volume = True,
+            flow_rate = flow_rate,
+        )
+
+        self._plot_base(grid, 'volumes_boxplot', 'plug volume boxplots')
+
+
     def sample_sd_violin(self):
 
         grid = self.plug_data.sample_sd_violin()
@@ -1148,14 +1391,14 @@ class PlugExperiment(object):
 
     def pmt_data_to_excel(
             self,
-            fname = None,
+            path = None,
             sheet_name = 'PMT_data',
             **kwargs,
         ):
 
         self._to_excel(
             df = self.pmt_data.data,
-            fname = fname,
+            path = path,
             sheet_name = sheet_name,
             fname_suffix = 'pmt',
             label = 'PMT data',
@@ -1165,14 +1408,14 @@ class PlugExperiment(object):
 
     def raw_plugs_to_excel(
             self,
-            fname = None,
+            path = None,
             sheet_name = 'Plugs_raw',
             **kwargs,
         ):
 
         self._to_excel(
             df = self.plug_data.plug_df,
-            fname = fname,
+            path = path,
             sheet_name = sheet_name,
             fname_suffix = 'plugs_raw',
             label = 'raw plug data',
@@ -1184,7 +1427,7 @@ class PlugExperiment(object):
 
         self._to_excel(
             df = self.plug_data.sample_df,
-            fname = fname,
+            path = path,
             sheet_name = sheet_name,
             fname_suffix = 'plugs',
             label = 'plug data',
@@ -1194,14 +1437,14 @@ class PlugExperiment(object):
 
     def samples_to_excel(
             self,
-            fname = None,
+            path = None,
             sheet_name = 'Samples',
             **kwargs,
         ):
 
         self._to_excel(
             df = self.sample_data,
-            fname = fname,
+            path = path,
             sheet_name = sheet_name,
             fname_suffix = 'samples',
             label = 'sample data',
@@ -1211,14 +1454,18 @@ class PlugExperiment(object):
 
     def stats_to_excel(
             self,
-            fname = None,
+            path = None,
             sheet_name = 'Statistics',
             **kwargs,
         ):
 
+        if not hasattr(self, 'sample_statistics'):
+
+            self.sample_statistics = self.stats()
+
         self._to_excel(
             df = self.sample_statistics,
-            fname = fname,
+            path = path,
             sheet_name = sheet_name,
             fname_suffix = 'stats',
             label = 'sample statistics',
@@ -1226,24 +1473,101 @@ class PlugExperiment(object):
         )
 
 
+    def _export(
+            self,
+            df,
+            path = None,
+            fname_suffix = None,
+            ftype: str = 'tsv',
+            label = 'data',
+            sheet_name = 'Sheet1',
+            **kwargs,
+        ):
+
+        methods = {
+            'tsv': 'csv',
+            'xlsx': 'excel',
+            'tex': 'latex',
+            'md': 'markdown',
+        }
+
+        ftype = ftype.lower()
+        path = path or self.config.result_dir
+
+        if os.path.isdir(path):
+
+            fname_suffix = fname_suffix or 'untitled'
+            fname = f'{self.pmt_data.input_file.stem}_{fname_suffix}.{ftype}'
+            path = os.path.join(path, fname)
+
+        if not os.path.dirname(path):
+
+            path = os.path.join(self.config.result_dir, path)
+
+
+        if 'index' not in kwargs:
+
+            kwargs['index'] = isinstance(df.index, pd.MultiIndex)
+
+        if ftype == 'tsv':
+
+            kwargs['sep'] = '\t'
+
+        if ftype == 'xlsx':
+
+            kwargs['sheet_name'] = sheet_name
+
+        method = f'to_{methods.get(ftype, ftype)}'
+
+        getattr(df, method)(path, **kwargs)
+
+        module_logger.info(f'Exported {label} to `{path}`.')
+
+
     def _to_excel(
             self,
             df,
-            fname = None,
+            path = None,
             fname_suffix = None,
             label = 'data',
             sheet_name = 'Sheet1',
             **kwargs,
         ):
 
-        fname = (
-            fname or
-            '%s_%s.xlsx' % (self.pmt_data.input_file.stem, fname_suffix)
-        )
+        args = locals()
+        args.pop('self')
+        args.pop('kwargs')
 
-        if 'index' not in kwargs:
+        self._export(ftype = 'xlsx', **args, **kwargs)
 
-            kwargs['index'] = isinstance(df.index, pd.MultiIndex)
 
-        df.to_excel(fname, sheet_name = sheet_name, **kwargs)
-        module_logger.info('Exported %s to %s' % (label, fname))
\ No newline at end of file
+    def _to_tsv(
+            self,
+            df,
+            path = None,
+            fname_suffix = None,
+            label = 'data',
+            **kwargs,
+        ):
+
+        args = locals()
+        args.pop('self')
+        args.pop('kwargs')
+
+        self._export(ftype = 'tsv', **args, **kwargs)
+
+
+    def _to_csv(
+            self,
+            df,
+            path = None,
+            fname_suffix = None,
+            label = 'data',
+            **kwargs,
+        ):
+
+        args = locals()
+        args.pop('self')
+        args.pop('kwargs')
+
+        self._export(ftype = 'csv', **args, **kwargs)
